# Generated by Django 5.0.9 on 2025-05-21 13:24

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_oemof", "0005_alter_result_name"),
    ]

    operations = [
        migrations.AddField(
            model_name="oemofdataset",
            name="meta_results",
            field=models.JSONField(default=dict),
            preserve_default=False,
        ),
    ]
