# Generated by Django 3.2.19 on 2023-06-16 09:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_oemof', '0004_alter_simulation_dataset'),
    ]

    operations = [
        migrations.AlterField(
            model_name='result',
            name='name',
            field=models.TextField(),
        ),
    ]
