# Generated by Django 3.2.16 on 2022-11-11 09:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_oemof', '0002_simulation'),
    ]

    operations = [
        migrations.CreateModel(
            name='Result',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('data_type', models.CharField(choices=[('series', 'series'), ('frame', 'frame')], max_length=9)),
                ('data', models.JSONField()),
                ('simulation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='results', to='django_oemof.simulation')),
            ],
        ),
    ]
