# Generated by Django 3.2.16 on 2023-01-06 03:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_oemof', '0003_result'),
    ]

    operations = [
        migrations.AlterField(
            model_name='simulation',
            name='dataset',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='django_oemof.oemofdataset'),
        ),
    ]
